/*
 * Access a calculation server.
 *
 * Copyright (C) 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>

#include <stdio.h>

#include "calc_client.h"
#include "calc_local.h"

int main(void)
{
  l4_cap_idx_t server = l4re_env_get_cap("server");

  /* Reference to a calculator and encapsulation. */

  ref_Calc ref = {.cap=server};
  Calc obj = {.ref=ref, .iface=&client_iface_Calc};

  /* Reference to another calculator. */

  ref_Calc local_ref = {.ptr=0};
  Calc local_obj = {.ref=local_ref, .iface=&local_iface_Calc};

  /* Call the calculator. */

  int result;

  /* Invoke the operation function directly. */

  Calc_add(ref, 123, 456, &result);
  printf("add 123, 456 -> %d\n", result);

  /* Invoke the operation via the interface. */

  obj.iface->add(obj.ref, 789, 123, &result);
  printf("add 789, 123 -> %d\n", result);

  /* Invoke an operation on the local object via the interface. */

  local_obj.iface->add(local_obj.ref, 789, 123, &result);
  printf("add 789, 123 -> %d\n", result);

  return 0;
}
